import * as THREE from "three";
import { Sky } from "three/examples/jsm/objects/Sky.js";
import { 
alertAsync
,confirmAsync 
,choiceAsync
,promptAsync
} from "./dialogUtil";
import { 
loadModel
,loadModelFromArrayBuffer
,loadPly
,loadPlyFromText
,makePlyGeometryFromText
,faceTo
} from "./modelUtil";
import { 
setup_default_ambient_light
,setup_default_directional_light
,setup_default_sky_and_sun
,prepare_bgm
,play_bgm
,stop_bgm
} from "./world";

//部屋の設定ファイル
//部屋追加する場合は
//1.roomconf.jsをコピーして部屋ページ用のroomconfを作る。
//2.記述を変更してその部屋の地形やギミックを定義する。
//3.scene.htmlをコピーして部屋ページ用のhtmlを作る。
//4.html側の./roomconfのインポート元を1で作った実際の設定ファイルに変える。
//5.チャット入力でgoto:コマンドで3のhtmlを指定するとその部屋に移動するボタンを表示する。

//signaling_url シグナリングサーバのurl
//ローカルPC上だけで動かす場合は変更不要
//もしlan内で動かす場合はシグナリングサーバを動かす端末の
//実際のipを指定する。
//シグナリングサーバをsslモードで動かしている場合はwss://でホスト名を指定する
export const signaling_url = "ws://127.0.0.1:14514";


/*
ルームの設定はここに書く
geodataを編集して地形を設定する。
コードで地形配置をしたい場合はinit_geodataを編集する。
光源設定を変えたい場合はinit_light_settingsを編集する。
*/

//geodata 部屋に表示するオブジェクト定義の配列
/*
項目定義
type 種類
  plane 色だけ指定する平面
  box 色だけ指定する箱
  glb glbファイルによる3dモデル resourceでパス指定
  ply plyファイルによる3dモデル resourceでパス指定
    plyについてはMagicaVoxelでエクスポートしたもの前提

resource モデルデータ
  type=glb plyの場合のファイルの場所

color 色
  type=boxの場合の箱の色
  16進リテラルで表記 例 0xff0000

px py pz 位置
  オブジェクトの配置位置

sx sy sz 大きさ
  オブジェクトの大きさ
  planeの場合はsx szのみ有効
  boxの場合はサイズそのもの
  glb plyの場合は倍率

animation アニメーション
  glbのみ有効 autoにすると自動再生
  アニメーションありのglbはオブジェクトをobjとするとき
  obj.actions[]にアニメーション名をキーとしてAnimationActionが積まれる
  それに対して play/stopメソッドで操作できる。
  例 obj["Take 001"].play()

collision 物理判定
  wall 侵入禁止
  none 判定なし
  そのほか 移動可能地形（階段とか）

rx ry rz 3dモデルの向き
  type=boxの場合は無効
  ※物理演算ではなく矩形範囲による疑似判定なので0 0 0以外は見た目と異なる場合がある

onSelectDown ロングタップのイベントハンドラ
  オブジェクトをロングタップした場合に呼ばれる
  引数はオブジェクトのインスタンス

onLoad 生成時のイベントハンドラ
  引数はオブジェクトのインスタンス
  

*/


export var geodata = [
  {
    type:"plane",
    name:"ground",
    px:0,
    py:0,
    pz:0,
    sx:50,
    sy:1,
    sz:50,
    color:0x5e5e5e,
    collision:"land",
    onLoad:async (obj)=>{
      const gridHelper = new THREE.GridHelper(50, 50);
      obj.parent.add(gridHelper);
    }
  },
/* 地形にロングタップイベントを仕込む 例 */
/*
  {
    type:"box",
    px:5,
    py:0.5,
    pz:0,
    sx:1,
    sy:1,
    sz:1,
    color:0xff0000,
    collision:"wall",
    onSelectDown:async ()=>{
      await alertAsync("こいつは障害物です");
    }
  },
*/
/* glbを表示する例 */
/*
  {
    type:"glb",
    px:0,
    py:0,
    pz:-5,
    sx:0.1,
    sy:0.1,
    sz:0.1,
    rx:0,
    ry:0,
    rz:0,
    resource:"asset/model/an_animated_cat.glb",
    color:0xffffff,
    collision:"wall",
    animation:"",
    onSelectDown:async (obj)=>{
      faceTo(obj,window.myplayer,0);
      console.log(obj.actions['Take 001']);
      if(obj.actions['Take 001'].isRunning()){
        await alertAsync("にゃ？");
        obj.actions['Take 001'].stop();
      }else{
        await alertAsync("にゃーん");
        obj.actions['Take 001'].play();
      }
      //window.the_cat = obj;
      
    }
  },
*/
/* plyを表示する例 */
/*
  {
    type:"ply",
    px:0,
    py:0,
    pz:5,
    sx:1,
    sy:1,
    sz:1,
    rx:0,
    ry:0,
    rz:0,
    resource:"asset/model/test.ply",
    color:0xffffff,
    collision:"wall",
    onSelectDown:async (obj)=>{
      if(!obj.tap_count){
        obj.tap_count=1;
      };
      faceTo(obj,window.myplayer,Math.PI/2);

      if(obj.tap_count==1){
        await alertAsync("初めまして");
      }else if(obj.tap_count==2){
        await alertAsync("また会いましたね");
      }else{
        await alertAsync("こんにちは");
      }
      obj.tap_count+=1;
    }
  }
*/
];

//init_geodata 地形定義初期化処理
/*
もしコードで地形データ生成したい場合はここでgeodataに追加する
*/
export const init_geodata = () =>{
  /*
  //例 螺旋階段追加
  const spiral_stairs = make_spiral_step_stairs_gdata();
  geodata = geodata.concat(spiral_stairs);
  */

}

//螺旋階段用のgdataを作るコード例
const make_spiral_step_stairs_gdata = (_param) =>{
  var param = {radius:10,size:1,step_y:0.25,step_a:Math.PI/32,phase:0,num:200,cx:0,cy:0,cz:0,color:0xff00ff};
  if(typeof(_param)== "object"){
    for(const k in _param){
      param[k] = _param[k];
    }
  }
  const r = param.radius;
  const cx = param.cx;
  const cy = param.cy;
  const cz = param.cz;
  const size = param.size;
  const color = param.color;
  const step_y = param.step_y;
  const step_a = param.step_a;
  var angle = param.phase;
  const num = param.num;
  var arr_gd = [];
  for(let i=0;i<num;i++){
    let px = cx + Math.cos(angle)*r;
    let py = cy + i * step_y;
    let pz = cz + Math.sin(angle)*r;
    angle += step_a;
    const gd = {
      px:px,
      py:py,
      pz:pz,
      sx:size,
      sy:0.25,
      sz:size,
      type:"box",
      color:color,
    };
    arr_gd.push(gd);
  }
  return arr_gd;
}

//ギミック処理
//マスターからのギミック通知が来た場合ここで解決する
//ギミック通知は舞台装置担当者への指示みたいなもん
//主な方法は
//1.init_light_settingsでオブジェクトを仕込んでおいてそれを操作する。
//2.geodataで読み込んだオブジェクトを操作する。
//  この場合、geodataにonLoadを設定してオブジェクトを変数に保持しておく
//配付できないリソースを使っているコードは無効化しているので
//使いたい場合は自前でファイルを用意してコードを書き換えて有効化してください。

export const resolve_gimmick = async (code,target)=>{
  //code 処理コード
  //target 対象プレイヤーid
  //alert("ギミック要求 code="+code);
  //ギミック例
  if(code=="water_0"){
    window.stop_water();//水面非表示
  }else if(code=="water_1"){
    window.play_water(0.3);//水面表示 引数は高さ
  }else if(code=="water_2"){
    window.play_water(0.6);//水面表示 引数は高さ
  }else if(code=="snow_0"){
    window.stop_snow();//雪非表示
  }else if(code=="snow_1"){
    window.play_snow();//雪表示
  }else if(code=="sky_0"){
    window.set_sky_rayleigh(0);//空の明るさ
  }else if(code=="sky_1"){
    window.set_sky_rayleigh(1);
  }else if(code=="sky_2"){
    window.set_sky_rayleigh(2);
  }else if(code=="sun_0"){
    window.set_sun_height(0);//太陽の高さ
  }else if(code=="sun_-1"){
    window.set_sun_height(-1);
  }else if(code=="sun_1"){
    window.set_sun_height(1);
  }else if(code=="hdri_0"){
    window.hide_hdri();
    window.show_sky();
  }else if(code=="hdri_1"){
    //window.hide_sky();
    //window.show_hdri("asset/texture/Factory_Catwalk_2k.hdr");//キューブマップ背景の設定
  }else if(code=="hdri_2"){
    //window.hide_sky();
    //window.show_hdri("asset/texture/Milkyway_small.hdr");
  }else if(code=="hdri_3"){
    //window.hide_sky();
    //window.show_hdri("asset/texture/Serpentine_Valley_3k.hdr");
  }else if(code=="hdri_4"){
    //window.hide_sky();
    //window.show_hdri("asset/texture/snowy_forest_path_02_2k.hdr");
  }else if(code=="bgm_1_start"){
    //play_bgm("__bgm_1");//bgm再生開始
  }else if(code=="bgm_1_end"){
    //stop_bgm("__bgm_1");//bgm再生停止
  }else if(code=="bgm_2_start"){
    //play_bgm("__bgm_2");
  }else if(code=="bgm_2_end"){
    //stop_bgm("__bgm_2");
  }else if(code=="bgm_3_start"){
    //play_bgm("__bgm_3");
  }else if(code=="bgm_3_end"){
    //stop_bgm("__bgm_3");

  }else if(code=="d_light_0"){
    window.set_d_light_intensity(0);//直射光強度指定
  }else if(code=="d_light_1"){
    window.set_d_light_intensity(1);//直射光強度指定
  }else if(code=="d_light_2"){
    window.set_d_light_intensity(2);//直射光強度指定
  }else if(code=="d_light_white"){
    window.set_d_light_color(0xffffff);//直射光色指定
  }else if(code=="d_light_red"){
    window.set_d_light_color(0xff0000);//直射光色指定
  }else if(code=="d_light_blue"){
    window.set_d_light_color(0x0000ff);//直射光色指定
  }else if(code=="a_light_0"){
    window.set_a_light_intensity(0);//環境光強度指定
  }else if(code=="a_light_1"){
    window.set_a_light_intensity(1);//環境光強度指定
  }else if(code=="a_light_2"){
    window.set_a_light_intensity(2);//環境光強度指定
  }else if(code=="a_light_white"){
    window.set_a_light_color(0xffffff);//環境光色指定
  }else if(code=="a_light_red"){
    window.set_a_light_color(0xff0000);//環境光色指定
  }else if(code=="a_light_blue"){
    window.set_a_light_color(0x0000ff);//環境光色指定


  }else if(code=="spotlight_on"){
    //対象プレイヤーの頭上にspotlightを付ける
    //すでにある場合は
    let tgt ;
    if(target in window.dic_player){
      tgt = window.dic_player[target];
    }else if(target == window.p2p_client.my_id){
      tgt = window.myplayer;
    }else{
      return;
    }
    const origin = {
      x:tgt.position.x,
      y:tgt.position.y+3.0,
      z:tgt.position.z,
    };
    if(!window.__gimmick_spotlight){
      window.__gimmick_spotlight={};
    };
    if(target in window.__gimmick_spotlight){
      //すでにある場合
      //位置更新して点灯
      window.__gimmick_spotlight[target].position.set(origin.x,origin.y,origin.z);
      window.__gimmick_spotlight[target].intensity = 10;
      window.__gimmick_spotlight[target].penumbra=0;
    }else{
      //ない場合は作る
      window.__gimmick_spotlight[target]=window.create_spotlight(origin,tgt,10);
    };
  }else if(code=="spotlight_off"){
    let tgt ;
    if(target in window.dic_player){
      tgt = window.dic_player[target];
    }else if(target == window.p2p_client.my_id){
      tgt = window.myplayer;
    }else{
      return;
    }
    if(!window.__gimmick_spotlight){
      window.__gimmick_spotlight={};
    };
    if(target in window.__gimmick_spotlight){
      //すでにある場合
      //intensityを0にする
      window.__gimmick_spotlight[target].intensity = 0;
    }else{
      //ない場合は何もしない
    };
  }else if(code=="stair"){
    if("stair" in window.dic_called_gimmick){
      //1回だけ実行してほしい場合は辞書に積んで弾く
    }else{
      //螺旋階段の動的追加
      const stair = make_spiral_step_stairs_gdata();
      window.append_box_land(stair);
      window.dic_called_gimmick["stair"]=1;
    }

  }
};

//init_light_settings 光源初期化
/*
  光源や空などの定義を変更したい場合はここをいじる
*/
export const init_light_settings = (scene)=>{
  //環境光
  setup_default_ambient_light(scene);
  //直線光
  setup_default_directional_light(scene);
  //空
  setup_default_sky_and_sun(scene);
  
  //bgm読み込みの例
  //prepare_bgm("__bgm_1","asset/bgm/chess.mp3")
  //prepare_bgm("__bgm_2","asset/bgm/seishishitauchu.mp3")
  //prepare_bgm("__bgm_3","asset/bgm/silence.mp3")
  
}




